'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (dataFactory) {
    return function (Component) {
        var DataWatcher = (function (_React$Component) {
            (0, _inherits3.default)(DataWatcher, _React$Component);

            function DataWatcher(props, context) {
                (0, _classCallCheck3.default)(this, DataWatcher);

                var _this = (0, _possibleConstructorReturn3.default)(this, Object.getPrototypeOf(DataWatcher).call(this, props, context));

                _this.cursors = {};
                _this.dataState = context.state;

                if (typeof _this.dataState === 'undefined') {
                    (0, _throwError2.default)('missing state. It can happen if you forgot to set state in DataInit\n                        or using any decorator before DataInit', _this.constructor.displayName);
                }

                _this._initCursors(props);

                _this.state = (0, _extends3.default)({}, _this.state, {
                    data: _this._getCurrentData()
                });

                _this._updateDataState = _this._updateDataState.bind(_this);
                return _this;
            }

            (0, _createClass3.default)(DataWatcher, [{
                key: 'componentDidMount',
                value: function componentDidMount() {
                    var _this2 = this;

                    (0, _keys2.default)(this.cursors).forEach(function (branch) {
                        _this2.dataState._addToWatchingQueue(_this2.cursors[branch].path);
                    });

                    this._updateDataState();
                    this._dataWatch();
                }
            }, {
                key: 'componentWillReceiveProps',
                value: function componentWillReceiveProps(nextProps) {
                    if (this._isChangedProps(nextProps)) {
                        this._reloadComponentData(nextProps);
                    }
                }
            }, {
                key: 'componentWillUnmount',
                value: function componentWillUnmount() {
                    var _this3 = this;

                    (0, _keys2.default)(this.cursors).forEach(function (branch) {
                        _this3.dataState._removeFromWatchingQueue(_this3.cursors[branch].path);
                    });

                    this._dataUnwatch();
                }
            }, {
                key: '_isChangedProps',
                value: function _isChangedProps(nextProps) {
                    var _this4 = this;

                    return (0, _keys2.default)(nextProps).some(function (key) {
                        return nextProps[key] !== _this4.props[key];
                    });
                }
            }, {
                key: '_reloadComponentData',
                value: function _reloadComponentData() {
                    var props = arguments.length <= 0 || arguments[0] === undefined ? this.props : arguments[0];

                    this._dataUnwatch();
                    this._initCursors(props);
                    this._updateDataState();
                    this._dataWatch();
                }
            }, {
                key: '_reloadComponentDataFromPath',
                value: function _reloadComponentDataFromPath() {
                    this._reloadComponentData();
                }
            }, {
                key: '_isValidObjectPathChunk',
                value: function _isValidObjectPathChunk(pathChunk) {
                    var _this5 = this;

                    return (0, _keys2.default)(pathChunk).every(function (key) {
                        return _this5._isValidPathChunk(pathChunk[key]);
                    });
                }
            }, {
                key: '_isValidPathChunk',
                value: function _isValidPathChunk(pathChunk) {
                    return typeof pathChunk !== 'undefined';
                }
            }, {
                key: '_isValidPath',
                value: function _isValidPath(path) {
                    var _this6 = this;

                    return path.length !== 0 && path.every(function (pathChunk) {
                        if ((typeof pathChunk === 'undefined' ? 'undefined' : (0, _typeof3.default)(pathChunk)) === 'object') {
                            return _this6._isValidObjectPathChunk(pathChunk);
                        }

                        return _this6._isValidPathChunk(pathChunk);
                    });
                }
            }, {
                key: '_prepareArrayPathChunk',
                value: function _prepareArrayPathChunk(pathChunk) {
                    var _this7 = this;

                    var stateTree = this.dataState.getTree();
                    var preparedCursorPath = this._prepareCursorPath(pathChunk);

                    if (!this._isValidPath(preparedCursorPath)) {
                        return;
                    }

                    var pathChunkCursor = stateTree.select(preparedCursorPath);
                    var shouldListenForUpdate = pathChunkCursor.listeners('update').every(function (listener) {
                        return listener.scope !== _this7;
                    });

                    if (shouldListenForUpdate) {
                        pathChunkCursor.on('update', this._reloadComponentDataFromPath, {
                            scope: this
                        });
                    }

                    return pathChunkCursor.get();
                }
            }, {
                key: '_prepareObjectPathChunk',
                value: function _prepareObjectPathChunk(pathChunk) {
                    var _this8 = this;

                    var preparedPathChunk = {};

                    (0, _keys2.default)(pathChunk).forEach(function (key) {
                        preparedPathChunk[key] = _this8._preparePathChunk(pathChunk[key]);
                    });

                    return preparedPathChunk;
                }
            }, {
                key: '_preparePathChunk',
                value: function _preparePathChunk(pathChunk) {
                    if (Array.isArray(pathChunk)) {
                        return this._prepareArrayPathChunk(pathChunk);
                    } else if ((typeof pathChunk === 'undefined' ? 'undefined' : (0, _typeof3.default)(pathChunk)) === 'object') {
                        return this._prepareObjectPathChunk(pathChunk);
                    }

                    return pathChunk;
                }
            }, {
                key: '_prepareCursorPath',
                value: function _prepareCursorPath(path) {
                    return path.map(this._preparePathChunk.bind(this));
                }
            }, {
                key: '_prepareCursorPaths',
                value: function _prepareCursorPaths(props) {
                    var _this9 = this;

                    var cursorPaths = dataFactory(props);

                    (0, _keys2.default)(cursorPaths).forEach(function (branch) {
                        cursorPaths[branch] = _this9._prepareCursorPath(cursorPaths[branch]);
                    });

                    return cursorPaths;
                }
            }, {
                key: '_initCursors',
                value: function _initCursors(props) {
                    var _this10 = this;

                    var stateTree = this.dataState.getTree();

                    this.cursorPaths = this._prepareCursorPaths(props);

                    (0, _keys2.default)(this.cursorPaths).filter(function (branch) {
                        return _this10._isValidPath(_this10.cursorPaths[branch]);
                    }).forEach(function (branch) {
                        _this10.cursors[branch] = stateTree.select(_this10.cursorPaths[branch]);
                    });
                }
            }, {
                key: '_dataWatch',
                value: function _dataWatch() {
                    var _this11 = this;

                    (0, _keys2.default)(this.cursors).forEach(function (cursorType) {
                        return _this11.cursors[cursorType].on('update', _this11._updateDataState);
                    });
                }
            }, {
                key: '_dataUnwatch',
                value: function _dataUnwatch() {
                    var _this12 = this;

                    (0, _keys2.default)(this.cursors).forEach(function (cursorType) {
                        return _this12.cursors[cursorType].off('update', _this12._updateDataState);
                    });
                }
            }, {
                key: '_getCurrentData',
                value: function _getCurrentData() {
                    var _this13 = this;

                    var data = {};

                    (0, _keys2.default)(this.cursors).forEach(function (branch) {
                        data[branch] = _this13.cursors[branch].get();
                    });

                    return data;
                }
            }, {
                key: '_updateDataState',
                value: function _updateDataState() {
                    this.setState({
                        data: this._getCurrentData()
                    });
                }
            }, {
                key: 'render',
                value: function render() {
                    return _react2.default.createElement(Component, (0, _extends3.default)({}, this.props, this.state.data), this.props.children);
                }
            }]);
            return DataWatcher;
        })(_react2.default.Component);

        DataWatcher.contextTypes = {
            state: _react2.default.PropTypes.instanceOf(_state2.default)
        };

        return DataWatcher;
    };
};

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _throwError = require('./throw-error');

var _throwError2 = _interopRequireDefault(_throwError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }